/*
 * Copyright (c) 2008-2019, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.internal.management.dto;

import com.hazelcast.config.WANQueueFullBehavior;
import com.hazelcast.config.WanPublisherConfig;
import com.hazelcast.internal.json.JsonObject;
import com.hazelcast.test.HazelcastParallelClassRunner;
import com.hazelcast.test.annotation.ParallelTest;
import com.hazelcast.test.annotation.QuickTest;
import org.junit.Test;
import org.junit.experimental.categories.Category;
import org.junit.runner.RunWith;

import java.util.HashMap;
import java.util.Map;

import static org.junit.Assert.assertEquals;

@RunWith(HazelcastParallelClassRunner.class)
@Category({QuickTest.class, ParallelTest.class})
public class WanPublisherConfigDTOTest {

    @Test
    public void testSerialization() {
        Map<String, Comparable> properties = new HashMap<String, Comparable>();
        properties.put("key1", "value1");
        properties.put("key2", "value2");

        WanPublisherConfig expected = new WanPublisherConfig()
                .setGroupName("myGroupName")
                .setPublisherId("myPublisherId")
                .setQueueCapacity(23)
                .setClassName("myClassName")
                .setQueueFullBehavior(WANQueueFullBehavior.THROW_EXCEPTION)
                .setProperties(properties);

        WanPublisherConfigDTO dto = new WanPublisherConfigDTO(expected);

        JsonObject json = dto.toJson();
        WanPublisherConfigDTO deserialized = new WanPublisherConfigDTO(null);
        deserialized.fromJson(json);

        WanPublisherConfig actual = deserialized.getConfig();
        assertEquals(expected.getGroupName(), actual.getGroupName());
        assertEquals(expected.getPublisherId(), actual.getPublisherId());
        assertEquals(expected.getQueueCapacity(), actual.getQueueCapacity());
        assertEquals(expected.getClassName(), actual.getClassName());
        assertEquals(expected.getQueueFullBehavior(), actual.getQueueFullBehavior());
        assertEquals(expected.getProperties(), actual.getProperties());
    }
}
