/*
 * Copyright (c) 2008-2019, Hazelcast, Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hazelcast.nio.serialization;

import com.hazelcast.nio.ObjectDataInput;
import com.hazelcast.nio.ObjectDataOutput;

import java.io.IOException;

@SuppressWarnings("unused")
class DataDataSerializable implements DataSerializable {

    Data data;

    DataDataSerializable() {
    }

    DataDataSerializable(Data data) {
        this.data = data;
    }

    @Override
    public void writeData(ObjectDataOutput out) throws IOException {
        out.writeData(data);
    }

    @Override
    public void readData(ObjectDataInput in) throws IOException {
        data = in.readData();
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }

        DataDataSerializable that = (DataDataSerializable) o;
        if (data != null ? !data.equals(that.data) : that.data != null) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return data != null ? data.hashCode() : 0;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("DataDataSerializable{");
        sb.append("data=").append(data);
        sb.append('}');
        return sb.toString();
    }
}
